@@ -5,23 +5,25 @@
 
     The Python domain.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import re
 
 from docutils import nodes
-from docutils.parsers.rst import Directive, directives
+from docutils.parsers.rst import directives
 from six import iteritems
 
-from sphinx import addnodes
+from sphinx import addnodes, locale
+from sphinx.deprecation import DeprecatedDict, RemovedInSphinx30Warning
 from sphinx.directives import ObjectDescription
 from sphinx.domains import Domain, ObjType, Index
-from sphinx.locale import l_, _
+from sphinx.locale import _, __
 from sphinx.roles import XRefRole
 from sphinx.util import logging
 from sphinx.util.docfields import Field, GroupedField, TypedField
+from sphinx.util.docutils import SphinxDirective
 from sphinx.util.nodes import make_refnode
 
 if False:
@@ -44,6 +46,24 @@ py_sig_re = re.compile(
           ''', re.VERBOSE)
 
 
+pairindextypes = {
+    'module':    _('module'),
+    'keyword':   _('keyword'),
+    'operator':  _('operator'),
+    'object':    _('object'),
+    'exception': _('exception'),
+    'statement': _('statement'),
+    'builtin':   _('built-in function'),
+}  # Dict[unicode, unicode]
+
+locale.pairindextypes = DeprecatedDict(
+    pairindextypes,
+    'sphinx.locale.pairindextypes is deprecated. '
+    'Please use sphinx.domains.python.pairindextypes instead.',
+    RemovedInSphinx30Warning
+)
+
+
 def _pseudo_parse_arglist(signode, arglist):
     # type: (addnodes.desc_signature, unicode) -> None
     """"Parse" a list of arguments separated by commas.
@@ -138,7 +158,7 @@ class PyXrefMixin(object):
             if split_contnode:
                 contnode = nodes.Text(sub_target)
 
-            if delims_re.match(sub_target):  # type: ignore
+            if delims_re.match(sub_target):
                 results.append(contnode or innernode(sub_target, sub_target))
             else:
                 results.append(self.make_xref(rolename, domain, sub_target,
@@ -148,7 +168,15 @@ class PyXrefMixin(object):
 
 
 class PyField(PyXrefMixin, Field):
-    pass
+    def make_xref(self, rolename, domain, target,
+                  innernode=nodes.emphasis, contnode=None, env=None):
+        # type: (unicode, unicode, unicode, nodes.Node, nodes.Node, BuildEnvironment) ->  nodes.Node  # NOQA
+        if rolename == 'class' and target == 'None':
+            # None is not a type, so use obj role instead.
+            rolename = 'obj'
+
+        return super(PyField, self).make_xref(rolename, domain, target,
+                                              innernode, contnode, env)
 
 
 class PyGroupedField(PyXrefMixin, GroupedField):
@@ -181,21 +209,21 @@ class PyObject(ObjectDescription):
     }
 
     doc_field_types = [
-        PyTypedField('parameter', label=l_('Parameters'),
+        PyTypedField('parameter', label=_('Parameters'),
                      names=('param', 'parameter', 'arg', 'argument',
                             'keyword', 'kwarg', 'kwparam'),
                      typerolename='class', typenames=('paramtype', 'type'),
                      can_collapse=True),
-        PyTypedField('variable', label=l_('Variables'), rolename='obj',
+        PyTypedField('variable', label=_('Variables'), rolename='obj',
                      names=('var', 'ivar', 'cvar'),
                      typerolename='class', typenames=('vartype',),
                      can_collapse=True),
-        PyGroupedField('exceptions', label=l_('Raises'), rolename='exc',
+        PyGroupedField('exceptions', label=_('Raises'), rolename='exc',
                        names=('raises', 'raise', 'exception', 'except'),
                        can_collapse=True),
-        Field('returnvalue', label=l_('Returns'), has_arg=False,
+        Field('returnvalue', label=_('Returns'), has_arg=False,
               names=('returns', 'return')),
-        PyField('returntype', label=l_('Return type'), has_arg=False,
+        PyField('returntype', label=_('Return type'), has_arg=False,
                 names=('rtype',), bodyrolename='class'),
     ]
 
@@ -225,7 +253,7 @@ class PyObject(ObjectDescription):
         * it is stripped from the displayed name if present
         * it is added to the full name (return value) if not present
         """
-        m = py_sig_re.match(sig)  # type: ignore
+        m = py_sig_re.match(sig)
         if m is None:
             raise ValueError
         name_prefix, name, arglist, retann = m.groups()
@@ -544,7 +572,7 @@ class PyDecoratorMethod(PyDecoratorMixin, PyClassmember):
         return PyClassmember.run(self)
 
 
-class PyModule(Directive):
+class PyModule(SphinxDirective):
     """
     Directive to mark description of a new module.
     """
@@ -562,19 +590,18 @@ class PyModule(Directive):
 
     def run(self):
         # type: () -> List[nodes.Node]
-        env = self.state.document.settings.env
         modname = self.arguments[0].strip()
         noindex = 'noindex' in self.options
-        env.ref_context['py:module'] = modname
+        self.env.ref_context['py:module'] = modname
         ret = []
         if not noindex:
-            env.domaindata['py']['modules'][modname] = (env.docname,
-                                                        self.options.get('synopsis', ''),
-                                                        self.options.get('platform', ''),
-                                                        'deprecated' in self.options)
+            self.env.domaindata['py']['modules'][modname] = (self.env.docname,
+                                                             self.options.get('synopsis', ''),
+                                                             self.options.get('platform', ''),
+                                                             'deprecated' in self.options)
             # make a duplicate entry in 'objects' to facilitate searching for
             # the module in PythonDomain.find_obj()
-            env.domaindata['py']['objects'][modname] = (env.docname, 'module')
+            self.env.domaindata['py']['objects'][modname] = (self.env.docname, 'module')
             targetnode = nodes.target('', '', ids=['module-' + modname],
                                       ismod=True)
             self.state.document.note_explicit_target(targetnode)
@@ -588,7 +615,7 @@ class PyModule(Directive):
         return ret
 
 
-class PyCurrentModule(Directive):
+class PyCurrentModule(SphinxDirective):
     """
     This directive is just to tell Sphinx that we're documenting
     stuff in module foo, but links to module foo won't lead here.
@@ -602,12 +629,11 @@ class PyCurrentModule(Directive):
 
     def run(self):
         # type: () -> List[nodes.Node]
-        env = self.state.document.settings.env
         modname = self.arguments[0].strip()
         if modname == 'None':
-            env.ref_context.pop('py:module', None)
+            self.env.ref_context.pop('py:module', None)
         else:
-            env.ref_context['py:module'] = modname
+            self.env.ref_context['py:module'] = modname
         return []
 
 
@@ -640,8 +666,8 @@ class PythonModuleIndex(Index):
     """
 
     name = 'modindex'
-    localname = l_('Python Module Index')
-    shortname = l_('modules')
+    localname = _('Python Module Index')
+    shortname = _('modules')
 
     def generate(self, docnames=None):
         # type: (Iterable[unicode]) -> Tuple[List[Tuple[unicode, List[List[Union[unicode, int]]]]], bool]  # NOQA
@@ -711,15 +737,15 @@ class PythonDomain(Domain):
     name = 'py'
     label = 'Python'
     object_types = {
-        'function':     ObjType(l_('function'),      'func', 'obj'),
-        'data':         ObjType(l_('data'),          'data', 'obj'),
-        'class':        ObjType(l_('class'),         'class', 'exc', 'obj'),
-        'exception':    ObjType(l_('exception'),     'exc', 'class', 'obj'),
-        'method':       ObjType(l_('method'),        'meth', 'obj'),
-        'classmethod':  ObjType(l_('class method'),  'meth', 'obj'),
-        'staticmethod': ObjType(l_('static method'), 'meth', 'obj'),
-        'attribute':    ObjType(l_('attribute'),     'attr', 'obj'),
-        'module':       ObjType(l_('module'),        'mod', 'obj'),
+        'function':     ObjType(_('function'),      'func', 'obj'),
+        'data':         ObjType(_('data'),          'data', 'obj'),
+        'class':        ObjType(_('class'),         'class', 'exc', 'obj'),
+        'exception':    ObjType(_('exception'),     'exc', 'class', 'obj'),
+        'method':       ObjType(_('method'),        'meth', 'obj'),
+        'classmethod':  ObjType(_('class method'),  'meth', 'obj'),
+        'staticmethod': ObjType(_('static method'), 'meth', 'obj'),
+        'attribute':    ObjType(_('attribute'),     'attr', 'obj'),
+        'module':       ObjType(_('module'),        'mod', 'obj'),
     }  # type: Dict[unicode, ObjType]
 
     directives = {
@@ -849,7 +875,7 @@ class PythonDomain(Domain):
         if not matches:
             return None
         elif len(matches) > 1:
-            logger.warning('more than one target found for cross-reference %r: %s',
+            logger.warning(__('more than one target found for cross-reference %r: %s'),
                            target, ', '.join(match[0] for match in matches),
                            type='ref', subtype='python', location=node)
         name, obj = matches[0]
@@ -920,6 +946,7 @@ def setup(app):
 
     return {
         'version': 'builtin',
+        'env_version': 1,
         'parallel_read_safe': True,
         'parallel_write_safe': True,
     }
