@@ -5,13 +5,12 @@
 
     Assets adapter for sphinx.environment.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-from typing import TYPE_CHECKING
-
-if TYPE_CHECKING:
+if False:
+    # For type annotation
     from sphinx.environment import BuildEnvironment  # NOQA
 
 
