@@ -5,7 +5,7 @@
 
     Highlight code blocks using Pygments.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -16,7 +16,7 @@ from pygments.filters import ErrorToken
 from pygments.formatters import HtmlFormatter, LatexFormatter
 from pygments.lexer import Lexer  # NOQA
 from pygments.lexers import get_lexer_by_name, guess_lexer
-from pygments.lexers import PythonLexer, Python3Lexer, PythonConsoleLexer, \
+from pygments.lexers import Python2Lexer, Python3Lexer, PythonConsoleLexer, \
     CLexer, TextLexer, RstLexer
 from pygments.styles import get_style_by_name
 from pygments.util import ClassNotFound
@@ -40,7 +40,7 @@ logger = logging.getLogger(__name__)
 
 lexers = dict(
     none = TextLexer(stripnl=False),
-    python = PythonLexer(stripnl=False),
+    python = Python2Lexer(stripnl=False),
     python3 = Python3Lexer(stripnl=False),
     pycon = PythonConsoleLexer(stripnl=False),
     pycon3 = PythonConsoleLexer(python3=True, stripnl=False),
@@ -91,7 +91,7 @@ class PygmentsBridge(object):
         self.trim_doctest_flags = trim_doctest_flags
         if trim_doctest_flags is not None:
             warnings.warn('trim_doctest_flags option for PygmentsBridge is now deprecated.',
-                          RemovedInSphinx30Warning)
+                          RemovedInSphinx30Warning, stacklevel=2)
 
     def get_formatter(self, **kwargs):
         # type: (Any) -> Formatter
@@ -101,7 +101,7 @@ class PygmentsBridge(object):
     def unhighlighted(self, source):
         # type: (unicode) -> unicode
         warnings.warn('PygmentsBridge.unhighlighted() is now deprecated.',
-                      RemovedInSphinx30Warning)
+                      RemovedInSphinx30Warning, stacklevel=2)
         if self.dest == 'html':
             return '<pre>' + htmlescape(source) + '</pre>\n'
         else:
@@ -149,8 +149,8 @@ class PygmentsBridge(object):
 
         # trim doctest options if wanted
         if isinstance(lexer, PythonConsoleLexer) and self.trim_doctest_flags:
-            source = doctest.blankline_re.sub('', source)  # type: ignore
-            source = doctest.doctestopt_re.sub('', source)  # type: ignore
+            source = doctest.blankline_re.sub('', source)
+            source = doctest.doctestopt_re.sub('', source)
 
         # highlight via Pygments
         formatter = self.get_formatter(**kwargs)
